#!/usr/bin/env node
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.StaticWebsite = void 0;
const aws_route53_1 = require("@aws-cdk/aws-route53");
const aws_s3_1 = require("@aws-cdk/aws-s3");
const aws_s3_deployment_1 = require("@aws-cdk/aws-s3-deployment");
const aws_certificatemanager_1 = require("@aws-cdk/aws-certificatemanager");
const core_1 = require("@aws-cdk/core");
const lib_1 = require("@aws-cdk/aws-route53-targets/lib");
const aws_iam_1 = require("@aws-cdk/aws-iam");
const aws_cloudfront_1 = require("@aws-cdk/aws-cloudfront");
/**
 * Static site infrastructure, which deploys site content to an S3 bucket.
 *
 * The site redirects from HTTP to HTTPS, using a CloudFront distribution,
 * Route53 alias record, and ACM certificate.
 */
class StaticWebsite extends core_1.Construct {
    constructor(parent, name, props) {
        super(parent, name);
        let zone;
        if (props.hostedZoneId) {
            zone = aws_route53_1.HostedZone.fromHostedZoneAttributes(this, 'Zone', {
                zoneName: props.domainName,
                hostedZoneId: props.hostedZoneId
            });
        }
        else {
            zone = aws_route53_1.HostedZone.fromLookup(this, 'Zone', {
                domainName: props.domainName,
            });
        }
        new core_1.CfnOutput(this, 'Site', { value: 'https://' + props.siteSubDomain + '.' + props.domainName });
        // Content bucket
        const siteBucket = new aws_s3_1.Bucket(this, 'SiteBucket', {
            bucketName: props.siteSubDomain + '.' + props.domainName,
            websiteIndexDocument: 'index.html',
            websiteErrorDocument: 'index.html',
            publicReadAccess: true,
            removalPolicy: core_1.RemovalPolicy.DESTROY,
        });
        new core_1.CfnOutput(this, 'Bucket', { value: siteBucket.bucketName });
        // TLS certificate
        const certificateArn = new aws_certificatemanager_1.DnsValidatedCertificate(this, 'SiteCertificate', {
            domainName: props.siteSubDomain + '.' + props.domainName,
            hostedZone: zone,
            region: 'us-east-1', // Cloudfront only checks this region for certificates.
        }).certificateArn;
        new core_1.CfnOutput(this, 'Certificate', { value: certificateArn });
        // CloudFront distribution that provides HTTPS
        const distribution = new aws_cloudfront_1.CloudFrontWebDistribution(this, 'SiteDistribution', {
            aliasConfiguration: {
                acmCertRef: certificateArn,
                names: [props.siteSubDomain + '.' + props.domainName],
                sslMethod: aws_cloudfront_1.SSLMethod.SNI,
                securityPolicy: aws_cloudfront_1.SecurityPolicyProtocol.TLS_V1_1_2016,
            },
            originConfigs: [
                {
                    customOriginSource: {
                        domainName: siteBucket.bucketWebsiteDomainName,
                        originProtocolPolicy: aws_cloudfront_1.OriginProtocolPolicy.HTTP_ONLY,
                    },
                    behaviors: [{ isDefaultBehavior: true }],
                }
            ]
        });
        new core_1.CfnOutput(this, 'DistributionId', { value: distribution.distributionId });
        // Route53 alias record for the CloudFront distribution
        new aws_route53_1.ARecord(this, 'SiteAliasRecord', {
            recordName: props.siteSubDomain + '.' + props.domainName,
            target: aws_route53_1.RecordTarget.fromAlias(new lib_1.CloudFrontTarget(distribution)),
            zone
        });
        const bucketDeploymentRole = new aws_iam_1.Role(this, 'BucketDeploymentRole', {
            assumedBy: new aws_iam_1.ServicePrincipal('lambda.amazonaws.com'),
            managedPolicies: [
                aws_iam_1.ManagedPolicy.fromManagedPolicyArn(this, "DevCdkBucketAccess", core_1.Fn.importValue(core_1.Fn.sub("service-framework-security-${Environment}-DevCdkBootStrapBucketManagedPolicyArn")))
            ]
        });
        // Deploy site contents to S3 bucket
        new aws_s3_deployment_1.BucketDeployment(this, 'DeployWithInvalidation', {
            sources: [aws_s3_deployment_1.Source.asset(props.assets)],
            destinationBucket: siteBucket,
            distribution,
            role: bucketDeploymentRole,
            distributionPaths: ['/*'],
        });
    }
}
exports.StaticWebsite = StaticWebsite;
//# sourceMappingURL=cdk-construct.js.map